package fct.ample.REVMLInterpretor.FMPUtils;





/**
 * Class FmpElements represents FMP Elements including the id, parent id, attributes and kind (solitary, root, etc).
 * @author Andre Sousa 
 */
public class FmpElement {
	static final long serialVersionUID = 3;

	private String id;
	private String parentId;
	private FeatureProperties properties;
	private FmpAttribute attribute;
	private int type;
	private String rootReference;

	/**
	 * The default constructor for FmpElement.
	 */
	public FmpElement() {
		this.id = new String();
		this.parentId = new String();
		this.properties = new FeatureProperties();
		this.attribute = null;
		this.type = -1;
		this.rootReference = null;
	}

	/**
	 * The constructor for FmpElement.
	 * @param id The ID of the element.
	 * @param parentId The ID of the element's parent.
	 * @param newAttributes The attributes of the element.
	 * @param type The type of the element (root, solitary, etc).
	 */
	public FmpElement(String id, String parentId, FeatureProperties properties, int type) {
		this.id = id;
		this.parentId = parentId;
		this.properties = properties;
		this.attribute = null;
		this.type = type;
		this.rootReference = null;
	}

	/**
	 * The constructor for FmpElement.
	 * @param id The ID of the element.
	 * @param parentId The ID of the element's parent.
	 * @param newAttributes The attributes of the element.
	 * @param type The type of the element (root, solitary, etc).
	 * @param state The state of the element.
	 */
	public FmpElement(String id, String parentId, FeatureProperties properties, FmpAttribute attribute, int type) {
		this.id = id;
		this.parentId = parentId;
		this.properties = properties;
		this.attribute = attribute;
		this.type = type;
		this.rootReference = null;
	}

	/**
	 * 
	 * @return
	 */
	public String getId() {
		return id;
	}

	/**
	 * 
	 * @param id
	 */
	public void setId(String id) {
		this.id = id;
	}

	/**
	 * 
	 * @return
	 */
	public String getParentId() {
		return parentId;
	}

	/**
	 * 
	 * @param parentId
	 */
	public void setParentId(String parentId) {
		this.parentId = parentId;
	}

	/**
	 * 
	 * @return
	 */
	public FeatureProperties getProperties() {
		return properties;
	}

	/**
	 * 
	 * @param properties
	 */
	public void setProperties(FeatureProperties properties) {
		this.properties = properties;
	}

	/**
	 * 
	 * @return
	 */
	public FmpAttribute getAttribute() {
		return attribute;
	}

	/**
	 * 
	 * @param attribute
	 */
	public void setAttribute(FmpAttribute attribute) {
		this.attribute = attribute;
	}

	/**
	 * 
	 * @return
	 */
	public int getType() {
		return type;
	}

	/**
	 * 
	 * @param type
	 */
	public void setType(int type) {
		this.type = type;
	}

	/**
	 * This returns the root ID for this reference element.
	 * @return A String representing the id of the root being referred.
	 */
	public String getRootReference() {
		return rootReference;
	}

	/**
	 * Sets the root feature of a reference. 
	 * @param rootReference A String with the ID of the root being referred.  
	 */
	public void setRootReference(String rootReference) {
		this.rootReference = rootReference;
	}

	public String toString() {
		String result = "id = " + id + "\n";
		result += "parentId = " + parentId + "\n";
		result += "type = " + typeToString() + "\n";
		result += "root reference = " + rootReference + "\n";
		result += properties.toString();
		return result;
	}

	/**
	 * 
	 */
	public void printFmpElement() {
		System.out.println();
		System.out.println(toString());
	}

	/**
	 * This converts the type of an element to a String.
	 * @return A String for textual representation of the element's type.  
	 */
	private String typeToString() {
		switch (type) {
		case Constants.ROOT_FEATURE_KIND:
			return "root feature";
		case Constants.SOLITARY_FEATURE_KIND:
			return "solitary feature";
		case Constants.FEATURE_GROUP_KIND:
			return "feature group";
		case Constants.GROUPED_FEATURE_KIND:
			return "grouped feature";
		case Constants.FEATURE_REFERENCE_KIND:
			return "feature reference";
		case Constants.FEATURE_MODEL_KIND:
			return "feature model";
		default:
			return Constants.UNDEFINED;
		}
	}
}