package fct.ample.REVMLInterpretor.FMPUtils;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.runtime.Platform;
import org.jdom.Attribute;
import org.jdom.DataConversionException;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.Namespace;
import org.jdom.input.SAXBuilder;




public class ImportFmpFeatures {
	


	//import Constants;

	/**
	 * This class is responsible for importing Feature models written in FMP format.
	 * @author Andre Sousa
	 */

		static final long serialVersionUID = 4;

		private Document document;

		/**
		 * The constructor for ImportFmpFile.
		 * @param fmpFile A String with the path for the FMP file to be imported.  
		 */

		public ImportFmpFeatures(String fmpFile) throws Exception {
			//System.out.println("importando modelos de features");
			
			 if (fmpFile.contains("platform:"))//path relativa
			{
				fmpFile = Platform.asLocalURL(new URL(fmpFile)).getFile();
			}
			
			//System.out.println("importando modelos de features2");
			File f = new File(fmpFile);
			//System.out.println("importando modelos de features3");
			this.document = this.loadDocument(f);
			//System.out.println("importando modelos de features-FIM");
		}

		/**
		 * This method loads the file into a JDOM document representing the FMP model.
		 * @param file The file to be read.
		 * @return A JDOM document representing the XML read from the file.
		 * @throws IOException
		 */
		private Document loadDocument(File file) throws Exception {
			SAXBuilder builder = new SAXBuilder();
			Document doc = new Document();
			
			try	{
				//System.out.println("importando modelos de features00");
				//System.out.println(file);
				//System.out.println(builder);
				doc = builder.build(file);
				//System.out.println("importando modelos de features11");
			}
			
			catch(JDOMException e) {
				throw new Exception(file.getName() + " does not contains well formed XML!\n" + e.getMessage());
			}
			//System.out.println("importando modelos de features22");
			return doc;
		}

		/**
		 * This function returns a Namespace object with the given String prefix.
		 * @param ns The prefix for the namespace.
		 * @return The Namespace represented by the prefix.
		 */
		private Namespace getFmpNamespace(String ns) {
			return document.getRootElement().getNamespace(ns);
		}

		/**
		 * This function returns an FeatureAttributes object containing the attributes (name, min, max) of a given feature.
		 * @param propertiesId The properties id of the feature for which we want to get the the attributes.
		 * @return A FeatureAttributes of the required feature, or null if there are none.
		 */
		private FeatureProperties getFmpFeatureProperties(String propertiesId) {
			try {
				Attribute name = new Attribute("name","null"), min = new Attribute("min","1"), max = new Attribute("max","1");
				Element metamodel, current, child;
				List<Element> children;
				Iterator<Element> it, j;

				metamodel = document.getRootElement().getChild("metaModel");
				//browse through all the <configuration> elements that are inside <metaModel> element.
				for(it = metamodel.getDescendants(new ElementNameFilter("configurations"));it.hasNext();) {
					current = it.next();
					//it is the node we were looking for.
					if(propertiesId.equals(current.getAttributeValue("id",getFmpNamespace("xmi")))) {
						children = current.getChildren("children");
						//go through all the children and get the values we need from each.
						for(j = children.iterator(); j.hasNext();) {
							child = j.next();
							if("name".equalsIgnoreCase(child.getAttributeValue("name")))
								name = child.getChild("typedValue").getAttribute("stringValue");
							else if("min".equalsIgnoreCase(child.getAttributeValue("name")))
								min = child.getChild("typedValue").getAttribute("integerValue");
							else if("max".equalsIgnoreCase(child.getAttributeValue("name")))
								max = child.getChild("typedValue").getAttribute("integerValue");
						}
						return new FeatureProperties(name.getValue(),min.getIntValue(),max.getIntValue());
					}
				}
				return null;
			}
			catch (DataConversionException e) {
				e.printStackTrace();
				return null;
			}
		}

		/**
		 * This function identifies the type of element that is represented by elem (root feature, solitary feature, feature group, etc). 
		 * @param elem the element node.
		 * @return An int representing the type of fmp element.
		 */	 
		private int getElementKind(Element elem) {
			Element parent = elem.getParentElement();
			String parentType = parent.getAttributeValue("type", getFmpNamespace("xsi"));
			String elemType = elem.getAttributeValue("type", getFmpNamespace("xsi"));
			if("model".equalsIgnoreCase(parent.getName()))
				return Constants.ROOT_FEATURE_KIND;
			if("fmp:Reference".equalsIgnoreCase(elemType))
				return Constants.FEATURE_REFERENCE_KIND;
			if("fmp:FeatureGroup".equalsIgnoreCase(elemType))
				return Constants.FEATURE_GROUP_KIND;
			if("fmp:Feature".equalsIgnoreCase(parentType))
				return Constants.SOLITARY_FEATURE_KIND;
			if("fmp:FeatureGroup".equalsIgnoreCase(parentType))
				return Constants.GROUPED_FEATURE_KIND;
			return Constants.INVALID_KIND;
		}

		/**
		 * This function searches for the ID of the element's parent node. 
		 * @param elem the element node.
		 * @return A String representing the ID of the parent node. 
		 */
		private String getParentId(Element elem) {
			Element parent = elem.getParentElement();
			String parentId = parent.getAttributeValue("id", getFmpNamespace("xmi"));
			if("model".equalsIgnoreCase(parent.getName()))
				return Constants.NOT_APPLICABLE;
			else
				return parentId;
		}

		/**
		 * This function searches for the ID of the root feature that is being pointed by reference with the given id. 
		 * @param id The id of the node.
		 * @return A string containing the ID of the root feature, or "<undefined>" if there is no root feature selected.
		 */
		private String getReferenceRootId(String id) {
			Element metamodel, current, child;
			List<Element> children;
			Iterator<Element> it, j;

			metamodel = document.getRootElement().getChild("metaModel");
			//browse through all the <configuration> elements that are inside <metaModel> element.
			for(it = metamodel.getDescendants(new ElementNameFilter("configurations"));it.hasNext();) {
				current = it.next();
				//it is the node we were looking for 
				if(id.equals(current.getAttributeValue("describedNode"))) {
					children = current.getChildren("children");
					//go through all the children and get the values we need
					for(j = children.iterator(); j.hasNext();) {
						child = j.next();
						if(child.getAttributeValue("feature") != null)
							return child.getAttributeValue("feature");
					}
				}
			}
			return Constants.UNDEFINED;
		}

		/**
		 * This method returns the attribute associated with a a given feature.  
		 * @param propertiesId the properties ID of the desired feature.
		 * @return An {@link FmpAttribute} representing the attributes, or null if there is none. 
		 */
		private FmpAttribute getFmpFeatureAttribute(String propertiesId) {
			Element metamodel, configurations, attribute, attribType;
			List<Element> configChildren, attribChilden;
			Iterator<Element> configIt, attribIt, typeIt;

			metamodel = document.getRootElement().getChild("metaModel");
			//browse through all the <configuration> elements that are inside <metaModel> element.
			for(configIt = metamodel.getDescendants(new ElementNameFilter("configurations"));configIt.hasNext();) {
				configurations = configIt.next();
				//it is the node we were looking for.
				if(propertiesId.equals(configurations.getAttributeValue("id",getFmpNamespace("xmi")))) {
					configChildren = configurations.getChildren("children");
					//go through all the children and search for the node we want.
					//the one with an attribute <name="Attribute"> and <state="USER_SELECTED">.
					for(attribIt = configChildren.iterator(); attribIt.hasNext();) {
						attribute = attribIt.next();
						if("attribute".equalsIgnoreCase(attribute.getAttributeValue("name")) && "USER_SELECTED".equalsIgnoreCase(attribute.getAttributeValue("state"))) {
							attribChilden = attribute.getChildren("children");
							attribChilden = attribChilden.get(0).getChildren("children");
							for(typeIt = attribChilden.iterator(); typeIt.hasNext();) {
								attribType = typeIt.next();
								if("NONE".equalsIgnoreCase(attribType.getAttributeValue("valueType")) && "USER_SELECTED".equalsIgnoreCase(attribType.getAttributeValue("state"))) {
									String id = attribType.getAttributeValue("id",getFmpNamespace("xmi"));
									String type = attribType.getAttributeValue("name");
									return new FmpAttribute(id,type,1,1);
								}
							}
						}
					}
				}
			}
			return null;
		}

		/**
		 * This function returns a list of all the elements present in a feature model.
		 * @return A list of all feature model elements, or an empty list if there are none.
		 */
		public List<FmpElement> getFeatureModelElements() {
			List<FmpElement> result = new ArrayList<FmpElement>();
			Element model = document.getRootElement().getChild("model");
			Element modelTemp = (Element)model.clone();
			List<Element> rootFeatures = modelTemp.getChildren();
			//remove the configurations of the model
			for(Iterator<Element> i = rootFeatures.iterator();i.hasNext();) {
				i.next().removeChildren("configurations");
			}
			for(Iterator<Element> j = modelTemp.getDescendants(new ElementNameFilter("children"));j.hasNext();) {
				Element current = j.next();
				String id = current.getAttributeValue("id",getFmpNamespace("xmi"));
				String propertiesId = current.getAttributeValue("properties");
				FmpElement feature = new FmpElement(id,getParentId(current),getFmpFeatureProperties(propertiesId),getElementKind(current));
				if(getElementKind(current) == Constants.FEATURE_REFERENCE_KIND) {
					feature.setRootReference(getReferenceRootId(id));
				}
				FmpAttribute attrib = getFmpFeatureAttribute(propertiesId);
				if (attrib != null) {
					feature.setAttribute(attrib);
				}
				result.add(feature);
			}
			return result;
		}


}
