
package org.ample.vml.vml4arch.util;

import java.util.List;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.EStructuralFeature;

public class ComparatorUtil {
	
	private static EPackage epVml4arch;
	private static EClass ecVariantOrdering;
	private static EClass ecVariant;
	private static EClass ecPair;
	private static EStructuralFeature esfVariantName;
	private static EStructuralFeature esfOrderingPairs;
	private static EStructuralFeature esfPairFirst;
	private static EStructuralFeature esfPairSecond;

	private static class UtilComparable implements Comparable {
		private EObject eoVariantOrdering;
		private EObject eoVariant;
		
		public UtilComparable (EObject eoVariantOrdering, EObject eoVariant) {
			this.eoVariant = eoVariant;
			this.eoVariantOrdering = eoVariantOrdering;
		}
		
		public EObject getVariant() {
			return eoVariant;
		}
		
		@Override
		public int compareTo(Object cmp) {
			if (cmp instanceof UtilComparable) {
				// We do not compare to anything else
				EObject eoCmp = ((UtilComparable) cmp).getVariant();
				return compareVariants (eoVariantOrdering, eoVariant, eoCmp);
			}
			return -1; // Always make us bigger than anything else, so that we get a chance to be compared to others at all
		}
		
		protected int compareVariants (EObject eoOrdering, EObject eoVariant, EObject eoCmpVariant) {
			if (eoCmpVariant.eIsSet(esfVariantName)) {
				// Only named variants could be given a special order...
				for (EObject eoPair: (List<EObject>)eoOrdering.eGet(esfOrderingPairs)) {
					if ((eoPair.eGet(esfPairFirst) == eoVariant) && (eoPair.eGet(esfPairSecond) == eoCmpVariant)) {
						return -1;
					} else 
					if ((eoPair.eGet(esfPairFirst) == eoCmpVariant) && (eoPair.eGet(esfPairSecond) == eoVariant)) {
						return 1;
					}
				}
			}
			return 0;
		}
	}
	
	public static Comparable getDefaultComparator() {
		return new Comparable() {
			@Override
			public int compareTo(Object arg0) {
				if (arg0 instanceof UtilComparable) {
					return 1; // Have to push those up, as otherwise they may not be compared to each other at all 
				}
				return 0;
			}};
	}
	
	private static void initialize() {
		if (epVml4arch != null) return; // Already initialized
		
		epVml4arch = EPackage.Registry.INSTANCE.getEPackage("http://www.ample.org/vml/vml4arch");
		if (epVml4arch == null) {
			// This shouldn't happen, as the metamodel should have been registered
			System.err.println("Couldn't access Vml4arch metamodel from registry!");
			return;
		}
		ecVariantOrdering = (EClass) epVml4arch.getEClassifier("VariantOrdering");
		ecVariant = (EClass) epVml4arch.getEClassifier("Variant");
		ecPair = (EClass) epVml4arch.getEClassifier("VarOrderPair");
		
		esfVariantName = ecVariant.getEStructuralFeature("name");
		esfOrderingPairs = ecVariantOrdering.getEStructuralFeature("pairs");
		esfPairFirst = ecPair.getEStructuralFeature("first");
		esfPairSecond = ecPair.getEStructuralFeature("second");
	}
	
	public static Comparable getComparatorFor(Object variantOrdering, Object variant) {
		initialize();
		return new UtilComparable ((EObject) variantOrdering, (EObject) variant);
	}
}