/**
 * <copyright>
 * </copyright>
 *
 * $Id$
 */
package org.ample.vmlLangInst.impl;

import java.io.IOException;
import java.net.URL;
import java.util.Collection;

import org.ample.vmlLangInst.ActionTransformation;
import org.ample.vmlLangInst.FeatureTransformation;
import org.ample.vmlLangInst.LangInstModel;
import org.ample.vmlLangInst.TransformationAspect;
import org.ample.vmlLangInst.VmlLangInstPackage;
import org.ample.vmlLangInst.util.EglHelper;
import org.ample.vmlstar.util.PluginHelper;
import org.ample.vmlstar.util.PluginHelper.PluginDescriptor;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.SubMonitor;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.impl.ENotificationImpl;
import org.eclipse.emf.ecore.impl.EObjectImpl;
import org.eclipse.emf.ecore.util.EObjectContainmentEList;
import org.eclipse.emf.ecore.util.InternalEList;
import org.eclipse.epsilon.egl.exceptions.EglRuntimeException;
import org.osgi.framework.BundleException;

/**
 * <!-- begin-user-doc --> An implementation of the model object '
 * <em><b>Transformation Aspect</b></em>'. <!-- end-user-doc -->
 * <p>
 * The following features are implemented:
 * <ul>
 *   <li>{@link org.ample.vmlLangInst.impl.TransformationAspectImpl#getFeatures <em>Features</em>}</li>
 *   <li>{@link org.ample.vmlLangInst.impl.TransformationAspectImpl#getTransformations <em>Transformations</em>}</li>
 * </ul>
 * </p>
 *
 * @generated
 */
public class TransformationAspectImpl extends EObjectImpl implements
		TransformationAspect {
	/**
	 * The cached value of the '{@link #getFeatures() <em>Features</em>}' containment reference.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @see #getFeatures()
	 * @generated
	 * @ordered
	 */
	protected FeatureTransformation features;
	/**
	 * The cached value of the '{@link #getTransformations()
	 * <em>Transformations</em>}' containment reference list. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @see #getTransformations()
	 * @generated
	 * @ordered
	 */
	protected EList<ActionTransformation> transformations;

	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
	protected TransformationAspectImpl() {
		super();
	}

	/**
	 * This is the operation that is really relevant here. It generates all the
	 * code needed for compiling a VML specification and for transforming models
	 * based on a VML specification and a product configuration.
	 * 
	 * @generated NOT
	 */
	public void generate(String projectName, SubMonitor progress) {
		try {
			LangInstModel lim = (LangInstModel) eContainer();
			IProject generatorProject = ResourcesPlugin.getWorkspace()
					.getRoot().getProject(projectName + ".generator");
			IProject editorProject = ResourcesPlugin.getWorkspace().getRoot()
					.getProject(projectName + ".editor");

			generateVMLCompiler(lim, editorProject, generatorProject, progress);
			generateProductConfigurer(lim, editorProject, generatorProject,
					progress);
			
			// Make sure packages are exported correctly.
			PluginHelper.addExportedPackage(generatorProject, "org.ample.vml." + lim.getName(), progress);			
			PluginHelper.addExportedPackage(generatorProject, "org.ample.vml." + lim.getName() + ".library", progress);
			// Make sure editor plugin correctly depends on generator plugin
			PluginHelper.addPluginDependencies(editorProject, true, progress, new PluginDescriptor(generatorProject.getName()));
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Generates all the code needed for compiling the scripts in the new VML
	 * instance into oAW transformation workflows.
	 * 
	 * @param lim
	 * @param editorProject
	 * @param generatorProject
	 * @param progress
	 * @throws IOException
	 * @throws EglRuntimeException
	 * @throws CoreException
	 * @throws BundleException
	 */
	private void generateVMLCompiler(LangInstModel lim, IProject editorProject,
			IProject generatorProject, SubMonitor progress) throws IOException,
			EglRuntimeException, CoreException, BundleException {
		/*
		 * TODO: What we still need to generate here...
		 * 
		 * - Actions in the editor project
		 * 
		 * - Action contributor in the editor project
		 * 
		 * - extensions into plugin.xml in the editor project
		 * 
		 * - additional required bundles in the editor project
		 * 
		 * - copy icons into the editor project
		 * 
		 * - Fixes in the generated generator scripts, required to adjust to
		 * different types of models, product configurations etc.
		 */

		// Generate compile.oaw
		URL eglURL = FileLocator
				.toFileURL(new URL(
						"platform:/plugin/org.ample.vmlstar.langinst.model/transformations/LIM2CompileOAW.egl"));
		IFolder compileFolder = generatorProject.getFolder(new Path(
				"/src/org/ample/vml/" + lim.getName()));
		IFile compileFile = compileFolder.getFile("compile.oaw");
		EglHelper
				.generateToFile(eglURL, lim, compileFile, progress.newChild(2));

		// Generate compiler.xpt
		eglURL = FileLocator
				.toFileURL(new URL(
						"platform:/plugin/org.ample.vmlstar.langinst.model/transformations/LIM2CompilerXPT.egl"));
		IFile compilerFile = compileFolder.getFile("compiler.xpt");
		EglHelper.generateToFile(eglURL, lim, compilerFile, progress
				.newChild(2));

		// Generate VML operators
		eglURL = FileLocator
				.toFileURL(new URL(
						"platform:/plugin/org.ample.vmlstar.langinst.model/transformations/LIM2OperatorsEXT.egl"));
		IFolder libraryFolder = compileFolder.getFolder("library");
		if (!libraryFolder.exists()) {
			libraryFolder.create(true, true, progress.newChild(1));
		}
		IFile operatorFile = libraryFolder.getFile(lim.getName()
				+ "Operators.ext");
		EglHelper.generateToFile(eglURL, lim, operatorFile, progress
				.newChild(2));

		// Generate VMLCompileAction.java
		eglURL = FileLocator
				.toFileURL(new URL(
						"platform:/plugin/org.ample.vmlstar.langinst.model/transformations/LIM2CompileActionJava.egl"));
		IFolder editorActionFolder = editorProject
				.getFolder("/src/org/ample/vml/" + lim.getName()
						+ "/editor/actions");
		if (!editorActionFolder.exists()) {
			editorActionFolder.create(true, true, progress.newChild(1));
		}
		IFile compileActionFile = editorActionFolder.getFile("Compile"
				+ firstToUpper(lim.getName()) + "Action.java");
		EglHelper.generateToFile(eglURL, lim, compileActionFile, progress
				.newChild(2));
		// addPluginDependencies(editorProject, true, progress.newChild(1), new
		// PluginDescriptor("org.ample.vmlstar.langinst.model"));

		// Generate plugin.xml
		eglURL = FileLocator
				.toFileURL(new URL(
						"platform:/plugin/org.ample.vmlstar.langinst.model/transformations/LIM2PluginXML.egl"));
		IFile pluginXMLFile = editorProject.getFile("plugin.xml");
		EglHelper.generateToFile(eglURL, lim, pluginXMLFile, progress
				.newChild(2));
	}

	/**
	 * Generates all the code needed for transforming models based on a product
	 * configuration
	 * 
	 * @param lim
	 * @param editorProject
	 * @param generatorProject
	 * @param progress
	 * @throws IOException
	 * @throws EglRuntimeException
	 * @throws CoreException
	 */
	private void generateProductConfigurer(LangInstModel lim,
			IProject editorProject, IProject generatorProject,
			SubMonitor progress) throws IOException, EglRuntimeException,
			CoreException {
		/*
		 * TODO What do we need to generate here? How to interface arbitrary
		 * product configurations?
		 */

		// Generate productConfiguration.ext
		URL eglURL = FileLocator
				.toFileURL(new URL(
						"platform:/plugin/org.ample.vmlstar.langinst.model/transformations/LIM2ProductConfigurationEXT.egl"));
		IFolder compileFolder = generatorProject.getFolder(new Path(
				"/src/org/ample/vml/" + lim.getName()));
		IFile compileFile = compileFolder.getFile("productConfiguration.ext");
		EglHelper
				.generateToFile(eglURL, lim, compileFile, progress.newChild(2));

		// Generate createExecutables.ext
		eglURL = FileLocator
				.toFileURL(new URL(
						"platform:/plugin/org.ample.vmlstar.langinst.model/transformations/LIM2CreateExecutableXPT.egl"));
		IFile createExecXPTFile = compileFolder
				.getFile("createExecutables.xpt");
		EglHelper.generateToFile(eglURL, lim, createExecXPTFile, progress
				.newChild(2));

		// Generate createExecutables.oaw
		eglURL = FileLocator
				.toFileURL(new URL(
						"platform:/plugin/org.ample.vmlstar.langinst.model/transformations/LIM2CreateExecutableOAW.egl"));
		IFile createExecOAWFile = compileFolder
				.getFile("createExecutables.oaw");
		EglHelper.generateToFile(eglURL, lim, createExecOAWFile, progress
				.newChild(2));

		// Generate ConfigureAction.java
		eglURL = FileLocator
				.toFileURL(new URL(
						"platform:/plugin/org.ample.vmlstar.langinst.model/transformations/LIM2ConfigureActionJava.egl"));
		IFolder editorActionFolder = editorProject
				.getFolder("/src/org/ample/vml/" + lim.getName()
						+ "/editor/actions");
		if (!editorActionFolder.exists()) {
			editorActionFolder.create(true, true, progress.newChild(1));
		}
		IFile configureActionFile = editorActionFolder.getFile("Configure"
				+ firstToUpper(lim.getName()) + "Action.java");
		EglHelper.generateToFile(eglURL, lim, configureActionFile, progress
				.newChild(2));

	}

	private String firstToUpper(String s) {
		char cFirstUpper = Character.toUpperCase(s.charAt(0));
		String sResult = cFirstUpper + s.substring(1);
		return sResult;
	}

	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	protected EClass eStaticClass() {
		return VmlLangInstPackage.Literals.TRANSFORMATION_ASPECT;
	}

	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
	public FeatureTransformation getFeatures() {
		return features;
	}

	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
	public NotificationChain basicSetFeatures(
			FeatureTransformation newFeatures, NotificationChain msgs) {
		FeatureTransformation oldFeatures = features;
		features = newFeatures;
		if (eNotificationRequired()) {
			ENotificationImpl notification = new ENotificationImpl(this, Notification.SET, VmlLangInstPackage.TRANSFORMATION_ASPECT__FEATURES, oldFeatures, newFeatures);
			if (msgs == null) msgs = notification; else msgs.add(notification);
		}
		return msgs;
	}

	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
	public void setFeatures(FeatureTransformation newFeatures) {
		if (newFeatures != features) {
			NotificationChain msgs = null;
			if (features != null)
				msgs = ((InternalEObject)features).eInverseRemove(this, EOPPOSITE_FEATURE_BASE - VmlLangInstPackage.TRANSFORMATION_ASPECT__FEATURES, null, msgs);
			if (newFeatures != null)
				msgs = ((InternalEObject)newFeatures).eInverseAdd(this, EOPPOSITE_FEATURE_BASE - VmlLangInstPackage.TRANSFORMATION_ASPECT__FEATURES, null, msgs);
			msgs = basicSetFeatures(newFeatures, msgs);
			if (msgs != null) msgs.dispatch();
		}
		else if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, VmlLangInstPackage.TRANSFORMATION_ASPECT__FEATURES, newFeatures, newFeatures));
	}

	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
	public EList<ActionTransformation> getTransformations() {
		if (transformations == null) {
			transformations = new EObjectContainmentEList<ActionTransformation>(ActionTransformation.class, this, VmlLangInstPackage.TRANSFORMATION_ASPECT__TRANSFORMATIONS);
		}
		return transformations;
	}

	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public NotificationChain eInverseRemove(InternalEObject otherEnd,
			int featureID, NotificationChain msgs) {
		switch (featureID) {
			case VmlLangInstPackage.TRANSFORMATION_ASPECT__FEATURES:
				return basicSetFeatures(null, msgs);
			case VmlLangInstPackage.TRANSFORMATION_ASPECT__TRANSFORMATIONS:
				return ((InternalEList<?>)getTransformations()).basicRemove(otherEnd, msgs);
		}
		return super.eInverseRemove(otherEnd, featureID, msgs);
	}

	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Object eGet(int featureID, boolean resolve, boolean coreType) {
		switch (featureID) {
			case VmlLangInstPackage.TRANSFORMATION_ASPECT__FEATURES:
				return getFeatures();
			case VmlLangInstPackage.TRANSFORMATION_ASPECT__TRANSFORMATIONS:
				return getTransformations();
		}
		return super.eGet(featureID, resolve, coreType);
	}

	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
	@SuppressWarnings("unchecked")
	@Override
	public void eSet(int featureID, Object newValue) {
		switch (featureID) {
			case VmlLangInstPackage.TRANSFORMATION_ASPECT__FEATURES:
				setFeatures((FeatureTransformation)newValue);
				return;
			case VmlLangInstPackage.TRANSFORMATION_ASPECT__TRANSFORMATIONS:
				getTransformations().clear();
				getTransformations().addAll((Collection<? extends ActionTransformation>)newValue);
				return;
		}
		super.eSet(featureID, newValue);
	}

	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public void eUnset(int featureID) {
		switch (featureID) {
			case VmlLangInstPackage.TRANSFORMATION_ASPECT__FEATURES:
				setFeatures((FeatureTransformation)null);
				return;
			case VmlLangInstPackage.TRANSFORMATION_ASPECT__TRANSFORMATIONS:
				getTransformations().clear();
				return;
		}
		super.eUnset(featureID);
	}

	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public boolean eIsSet(int featureID) {
		switch (featureID) {
			case VmlLangInstPackage.TRANSFORMATION_ASPECT__FEATURES:
				return features != null;
			case VmlLangInstPackage.TRANSFORMATION_ASPECT__TRANSFORMATIONS:
				return transformations != null && !transformations.isEmpty();
		}
		return super.eIsSet(featureID);
	}

} // TransformationAspectImpl
