package org.ample.vmlLangInst.util;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.net.URL;

import org.ample.vmlLangInst.LangInstModel;
import org.ample.vmlLangInst.VmlLangInstPackage;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.SubMonitor;
import org.eclipse.epsilon.commons.parse.problem.ParseProblem;
import org.eclipse.epsilon.egl.EglModule;
import org.eclipse.epsilon.egl.exceptions.EglRuntimeException;
import org.eclipse.epsilon.emc.emf.InMemoryEmfModel;

/**
 * Helper class with methods for generating from EGL files.
 * 
 * @author zschaler
 */
public class EglHelper {

	/**
	 * Generate a file based on a template and a LangInstModel. Assumes the
	 * submonitor has been set up for two units of work.
	 * 
	 * @param templateURL
	 * @param lim
	 * @param targetFile
	 * @param monitor
	 * @throws EglRuntimeException
	 * @throws IOException
	 * @throws CoreException
	 */
	public static void generateToFile(URL templateURL, LangInstModel lim,
			IFile targetFile, SubMonitor monitor) throws EglRuntimeException,
			IOException, CoreException {
		try {
			String sGenerated = generateToString(templateURL, lim);
			if (targetFile.exists()) {
				targetFile.delete(true, monitor.newChild(1));
			} else {
				monitor.worked(1);
			}
			targetFile.create(new ByteArrayInputStream(sGenerated.getBytes()),
					true, monitor.newChild(1));
		} finally {
			monitor.done();
		}
	}

	public static String generateToString(URL templateURL, LangInstModel lim)
			throws EglRuntimeException, IOException {
		EglModule module = new EglModule();
		try {
			java.net.URI eglURI = java.net.URI.create(templateURL.toString());
			module.parse(eglURI);
			if (module.getParseProblems().size() > 0) {
				System.err.println("Parse errors occured...");
				for (ParseProblem problem : module.getParseProblems()) {
					System.err.println(problem.toString());
				}
				throw new RuntimeException("Parse Problems.");
			}

			// Attach the LanguageInstanceModel
			InMemoryEmfModel limModel = new InMemoryEmfModel("LIM",
					LIMResourceManager.getResourceFor(lim),
					VmlLangInstPackage.eINSTANCE);
			module.getContext().getModelRepository().addModel(limModel);

			module.execute();

			return module.getContext().getOutputBuffer().toString();
		} finally {
			module.getContext().getModelRepository().dispose();
		}
	}
}