/**
 * 
 */
package org.ample.vmlLangInst.util;

import java.util.HashMap;
import java.util.Map;

import org.ample.vmlLangInst.LangInstModel;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;

/**
 * This class manages in-memory resources for LanguageInstModels as they are
 * used throughout the various generators.
 * 
 * @author zschaler
 */
public class LIMResourceManager {

	private static Map<LangInstModel, Resource> s_ResourceMap = new HashMap<LangInstModel, Resource>();
	
	public static Resource getResourceFor (LangInstModel lim) {
		Resource limResource = s_ResourceMap.get(lim);
		if (limResource == null) {
			limResource = putInResource (lim);
			s_ResourceMap.put (lim, limResource);
		}
		
		return limResource;
	}
	
	public static void removeResourceFor (LangInstModel lim) {
		s_ResourceMap.remove(lim);
	}
	
	private static Resource putInResource(EObject eObject) {
		ResourceSet rs = new ResourceSetImpl();
		EPackage ePackage = eObject.eClass().getEPackage();
		rs.getResourceFactoryRegistry().getExtensionToFactoryMap().put("*",
				new XMIResourceFactoryImpl());
		rs.getPackageRegistry().put(ePackage.getNsURI(), ePackage);
		Resource r = rs.createResource(URI.createFileURI("foo"));
		r.getContents().add(eObject);
		return r;
	}
}